﻿/* Copyright(c) 2009, Rubenhak
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   -> Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *   -> Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using System.Windows.Media.Animation;
using System.Windows;
using System.Windows.Data;
using System.Windows.Media;
using System.ComponentModel;

/*
 * This file is the part of Rubenhak.Common.WPF library.
 * New versions are be found at http://rubenhak.com/.
 */
namespace Rubenhak.Common.WPF
{
    #region Documentation Tags
    /// <summary>
    ///     A button which listens to its IsMouseOver, IsPressed and IsEnabled states and automaticly
    ///     runs corresponding animations.
    /// </summary>
    /// <remarks>
    /// <para>
    ///     Class Information:
    ///	    <list type="bullet">
    ///         <item name="authors">Authors: Ruben Hakopian</item>
    ///         <item name="date">April 2009</item>
    ///         <item name="originalURL">http://rubenhak.com/?p=113</item>
    ///     </list>
    /// </para>
    /// </remarks>
    #endregion
    public class AnimatableButton : Button
    {
        #region Private Members

        private Storyboard _currentAnimation = null;
        private Storyboard _currentTemplateAnimation = null;

        #endregion

        #region State Propery

        public ButtonState State
        {
            get { return (ButtonState)GetValue(StateProperty); }
            private set { SetValue(StateProperty, value); }
        }

        // Using a DependencyProperty as the backing store for State.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty StateProperty =
            DependencyProperty.Register("State", typeof(ButtonState), typeof(AnimatableButton), new UIPropertyMetadata(StateChangedCallback));

        private static void StateChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            AnimatableButton _this = (d as AnimatableButton);
            _this.InvalidateAnimation();
        }

        #endregion

        #region Constructors

        public AnimatableButton()
        {
            MultiBinding b = new MultiBinding();
            b.Converter = new StateConverter();
            b.Bindings.Add(new Binding("IsPressed") { Source = this });
            b.Bindings.Add(new Binding("IsMouseOver") { Source = this });
            b.Bindings.Add(new Binding("IsEnabled") { Source = this });
            this.SetBinding(AnimatableButton.StateProperty, b);

            this.Loaded += new RoutedEventHandler(AnimatableButton_Loaded);
        }

        #endregion

        #region Protected Methods

        protected void InvalidateAnimation()
        {
            if (DesignerProperties.GetIsInDesignMode(this))
                return;

            RunAnimation();
            RunTemplateAnimation();
        }

        protected virtual string GetCurrentAnimationName()
        {
            switch (State)
            {
                case ButtonState.Normal:
                    return "NormalAnimation";
                case ButtonState.MouseOver:
                    return "MouseOverAnimation";
                case ButtonState.Pressed:
                    return "PressedAnimation";
                case ButtonState.Disabled:
                    return "DisabledAnimation";
                default:
                    return string.Empty;
            }
        }

        #endregion

        #region Private Methods

        private void AnimatableButton_Loaded(object sender, RoutedEventArgs e)
        {
            InvalidateAnimation();
        }

        private void RunAnimation()
        {
            string animationName = GetCurrentAnimationName();

            Storyboard animation = this.Resources[animationName] as Storyboard;
            if (animation == null)
                return;

            if (_currentAnimation != null)
                _currentAnimation.Stop(this);

            _currentAnimation = animation;
            _currentAnimation.Begin(this, true);
        }

        private void RunTemplateAnimation()
        {
            if (this.Template == null)
                return;

            string animationName = GetCurrentAnimationName();

            Storyboard animation = this.Template.Resources[animationName] as Storyboard;
            if (animation == null)
                return;

            if (VisualTreeHelper.GetChildrenCount(this) == 0)
                return;

            FrameworkElement contentPresenter = VisualTreeHelper.GetChild(this, 0) as FrameworkElement;
            if (contentPresenter == null)
                return;

            if (_currentTemplateAnimation != null)
                _currentTemplateAnimation.Stop(contentPresenter);

            _currentTemplateAnimation = animation;
            _currentTemplateAnimation.Begin(contentPresenter, true);
        }

        #endregion

        #region Contained Classes

        private class StateConverter : IMultiValueConverter
        {
            public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
            {
                bool isPressed = (bool)values[0];
                bool isMouseOver = (bool)values[1];
                bool isEnabled = (bool)values[2];

                if (isEnabled)
                {
                    if (isPressed)
                    {
                        return ButtonState.Pressed;
                    }
                    else
                    {
                        if (isMouseOver)
                        {
                            return ButtonState.MouseOver;
                        }
                        else
                        {
                            return ButtonState.Normal;
                        }
                    }
                }
                else
                {
                    return ButtonState.Disabled;
                }
            }

            public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
            {
                throw new NotImplementedException();
            }
        }

        #endregion
    }

    /// <summary>
    /// Defines the state of the button.
    /// </summary>
    public enum ButtonState
    {
        Normal,
        MouseOver,
        Pressed,
        Disabled
    }
}
